{******************************************************************}
{                                                                  }
{ Borland Delphi APi for Windows Media Format 7.1                  }
{    Date  : 16-09-2001                                            }
{    Autor : Henri GOURVEST                                        }
{    Email : hgourvest@progdigy.com                                }
{    home  : http://www.progdigy.com                               }
{                                                                  }
{ Contributor: Thomas Schimming                                    }
{                                                                  }
{  Files: key.h, key.c, sha.h, sac.h, des.h.                       }
{                                                                  }
{ Portions created by Microsoft are                                }
{ Copyright (C) 1995-1999 Microsoft Corporation.                   }
{ All Rights Reserved.                                             }
{                                                                  }
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/MPL/MPL-1.1.html                          }
{                                                                  }
{ Software distributed under the License is distributed on an      }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License.                        }
{                                                                  }
{******************************************************************}

unit WMFCrypt;

interface

uses
  Windows, ActiveX;

//------------------------------------------------------------------------------
// key.h + key.c
//
// To interact with the WMDM, you need to obtain a valid certificate from Microsoft.
//
//------------------------------------------------------------------------------

const
  abPVK  = [$00]; // array of byte
  abCert = [$00]; // array of byte

//-----------------------------------------------------------------------------
//
// File:   sha.h
//
// Microsoft Digital Rights Management
// Copyright (C) Microsoft Corporation, 1999-2001. All Rights Reserved
//
// Description:
//
//-----------------------------------------------------------------------------

{  Copyright (C) RSA Data Security, Inc. created 1993.  This is an
   unpublished work protected as such under copyright law.  This work
   contains proprietary, confidential, and trade secret information of
   RSA Data Security, Inc.  Use, disclosure or reproduction without the
   express written authorization of RSA Data Security, Inc. is
   prohibited.
 }

const
  A_SHA_DIGEST_LEN = 20;

type
  PA_SHA_CTX = ^TA_SHA_CTX;
  TA_SHA_CTX = record
    FinishFlag : DWORD;
    HashVal    : array[0..A_SHA_DIGEST_LEN-1] of BYTE;
    state      : array[0..4] of DWORD;                   // state (ABCDE)
    count      : array[0..1] of DWORD;                    // number of bytes, msb first
    buffer     : array[0..63] of byte;                   // input buffer
  end;

  TA_SHA_DIGEST = array[0..A_SHA_DIGEST_LEN-1] of byte;

procedure A_SHAInit(v1: PA_SHA_CTX); stdcall;
procedure A_SHAUpdate(v1: PA_SHA_CTX; v2: pointer; v3: UINT); stdcall;
procedure A_SHAFinal(v1: PA_SHA_CTX; v2: TA_SHA_DIGEST); stdcall;

//
// versions that don't internally byteswap (NoSwap version), for apps like
// the RNG that don't need hash compatibility - perf increase helps.
//

procedure A_SHAUpdateNS(v1: PA_SHA_CTX; v2: pointer; v3: UINT); stdcall;
procedure A_SHAFinalNS(v1: PA_SHA_CTX; v2: TA_SHA_DIGEST); stdcall;

//------------------------------------------------------------------------------
// sac.h
//------------------------------------------------------------------------------

type
  THMAC = DWORD;

const
  RSA_KEY_LEN        = 64;
  SAC_SESSION_KEYLEN = 8;

  SAC_PROTOCOL_WMDM  = 1;
  SAC_PROTOCOL_V1    = 2;

  SAC_CERT_X509      = 1;
  SAC_CERT_V1        = 2;

type
  TMACINFO = record
    fUsed      : BOOL;
    abMacState : array[0..35] of BYTE;
  end;

//-----------------------------------------------------------------------------
//
// File:   des.h
//
// Microsoft Digital Rights Management
// Copyright (C) Microsoft Corporation, 1999-2001. All Rights Reserved
//
// Description:
//
//-----------------------------------------------------------------------------

type
  TDESTable = record
    keytab   : array[0..15,0..1] of  UINT;
  end;


const
  DES_TABLESIZE =  (sizeof(TDESTable));
  DES_BLOCKLEN  =  (8);
  DES_KEYSIZE   =  (8);

type
  PDESXTable = ^TDESXTable;
  TDESXTable = record
    inWhitening  : array[0..7] of byte;
    outWhitening : array[0..7] of byte;
    desTable     : TDESTable;
  end;

const
  DESX_TABLESIZE  = (sizeof(TDESXTable));
  DESX_BLOCKLEN   = (8);
  DESX_KEYSIZE    = (24);

{ In deskey.c:

     Fill in the DESTable struct with the decrypt and encrypt
     key expansions.

     Assumes that the second parameter points to DES_BLOCKLEN
     bytes of key.

}

procedure deskey(out v1: TDESTable; v2 : pointer); stdcall; // v2 = unsigned char

{ In desport.c:

     Encrypt or decrypt with the key in DESTable

}

procedure des(out pbOut; pbIn: pointer; key : pointer; op: integer); stdcall;

//
// set the parity on the DES key to be odd
// NOTE : must be called before deskey
// key must be cbKey number of bytes
//
procedure desparityonkey(pbKey: pointer; cbKey: DWORD); stdcall;

//
// reduce the DES key to a 40 bit key
// NOTE : must be called before deskey
// key must be 8 bytes
//
procedure desreducekey(key: pointer); stdcall;

// Expand 40 bit DES key to 64 and check weakness
// same as desreducekey except expands instead of weakening keys
procedure deskeyexpand(pbKey: pointer; out pbExpanded_key); stdcall;


procedure desexpand128to192(
    pbKey: pointer;        // input 128bit or 192bit buffer
    out pbExpandedKey      // output buffer (must be 192bit wide if pbKey == pbExpandedKey
    ); stdcall;

// DES-X routines

// initialize desX key struct.  key size is 24 bytes
procedure desxkey(k: PDESXTable; key: pointer); stdcall;

procedure desx(out pbOut; pbIn: pointer; keyin: pointer; op: integer); stdcall;

implementation

const
  WMLIB    = 'WMFLIB.DLL';
  DESXLIB    = 'WMFLIB.DLL';

procedure A_SHAInit(v1: PA_SHA_CTX); stdcall; external WMLIB;
procedure A_SHAUpdate(v1: PA_SHA_CTX; v2: pointer; v3: UINT); stdcall; external WMLIB;
procedure A_SHAFinal(v1: PA_SHA_CTX; v2: TA_SHA_DIGEST); stdcall; external WMLIB;
procedure A_SHAUpdateNS(v1: PA_SHA_CTX; v2: pointer; v3: UINT); stdcall; external WMLIB;
procedure A_SHAFinalNS(v1: PA_SHA_CTX; v2: TA_SHA_DIGEST); stdcall; external WMLIB;

procedure deskey(out v1: TDESTable; v2 : pointer); stdcall; external WMLIB; // v2 = unsigned char
procedure des(out pbOut; pbIn: pointer; key : pointer; op: integer); stdcall; external WMLIB;
procedure desparityonkey(pbKey: pointer; cbKey: DWORD); stdcall; external WMLIB;
procedure desreducekey(key: pointer); stdcall; external WMLIB;
procedure deskeyexpand(pbKey: pointer; out pbExpanded_key); stdcall; external WMLIB;
procedure desexpand128to192(pbKey: pointer; out pbExpandedKey); stdcall; external WMLIB;
procedure desxkey(k: PDESXTable; key: pointer); stdcall; external DESXLIB;
procedure desx(out pbOut; pbIn: pointer; keyin: pointer; op: integer); stdcall; external DESXLIB;

end.
